@@ -34,6 +34,7 @@ pub type OfferId = [u8; Nonce::BYTES];
 mod attribute;
 mod attribute_schema;
 mod attribute_type;
+mod entity_credentials;
 mod error;
 mod ext;
 mod fragment1;
@@ -43,15 +44,19 @@ mod messages;
 mod offer;
 mod presentation;
 mod presentation_manifest;
+mod profile_credentials;
+mod profile_state_credentials;
 mod request;
 mod schema;
 mod traits;
+mod traits_credentials;
 mod util;
 mod workers;
 
 pub use attribute::*;
 pub use attribute_schema::*;
 pub use attribute_type::*;
+pub use entity_credentials::*;
 pub use error::*;
 pub use ext::*;
 pub use fragment1::*;
@@ -61,12 +66,15 @@ pub(crate) use messages::*;
 pub use offer::*;
 pub use presentation::*;
 pub use presentation_manifest::*;
+pub use profile_credentials::*;
+pub use profile_state_credentials::*;
 pub use request::*;
 pub use schema::*;
 use serde::{Deserialize, Serialize};
 use signature_bbs_plus::Signature;
 use signature_core::nonce::Nonce;
 pub use traits::*;
+pub use traits_credentials::*;
 use util::*;
 pub(crate) use workers::*;
 
