@@ -4,8 +4,8 @@ use core::convert::TryFrom;
 use core::fmt::{Display, Formatter};
 use ockam_core::compat::string::String;
 use ockam_core::hex::encode;
+use ockam_core::vault::{Hasher, KeyId};
 use ockam_core::{Error, Result};
-use ockam_vault_core::{Hasher, KeyId};
 use serde::{Deserialize, Serialize};
 
 /// An identifier of a Profile.
