@@ -4,7 +4,7 @@ use crate::{
     profile::Profile,
     AuthenticationProof, Changes, Contact, Contacts, EntityError,
     EntityError::{ContactVerificationFailed, InvalidInternalState},
-    EventIdentifier, Identity, KeyAttributes, Lease, MetaKeyAttributes, ProfileChangeEvent,
+    EventIdentifier, KeyAttributes, Lease, MetaKeyAttributes, ProfileChangeEvent,
     ProfileEventAttributes, ProfileIdentifier, ProfileVault, TTL,
 };
 use cfg_if::cfg_if;
@@ -13,10 +13,9 @@ use ockam_core::compat::{
     string::{String, ToString},
     vec::Vec,
 };
+use ockam_core::vault::{SecretPersistence, SecretType, SecretVault, CURVE25519_SECRET_LENGTH};
 use ockam_core::{allow, deny, Address, AsyncTryClone, Result, Route};
-use ockam_core::{async_trait, compat::boxed::Box};
 use ockam_vault::{KeyIdVault, PublicKey, Secret, SecretAttributes};
-use ockam_vault_core::{SecretPersistence, SecretType, CURVE25519_SECRET_LENGTH};
 use ockam_vault_sync_core::VaultSync;
 
 cfg_if! {
@@ -27,7 +26,6 @@ cfg_if! {
 }
 
 /// Profile implementation
-#[derive(AsyncTryClone)]
 pub struct ProfileState {
     id: ProfileIdentifier,
     change_history: ProfileChangeHistory,
@@ -76,16 +74,17 @@ impl ProfileState {
     pub(crate) async fn create(mut vault: VaultSync) -> Result<Self> {
         let initial_event_id = EventIdentifier::initial(&mut vault).await;
 
-        let key_attribs = KeyAttributes::with_attributes(
-            Profile::PROFILE_UPDATE.to_string(),
+        let key_attribs = KeyAttributes::new(
+            Profile::ROOT_LABEL.to_string(),
             MetaKeyAttributes::SecretAttributes(SecretAttributes::new(
-                SecretType::Curve25519,
+                SecretType::Ed25519,
                 SecretPersistence::Persistent,
                 CURVE25519_SECRET_LENGTH,
             )),
         );
 
-        let create_key_event = Self::create_key_static(
+        let create_key_event = Self::make_create_key_event_static(
+            None,
             initial_event_id,
             key_attribs.clone(),
             ProfileEventAttributes::new(),
@@ -94,11 +93,7 @@ impl ProfileState {
         )
         .await?;
 
-        let create_key_change =
-            ProfileChangeHistory::find_key_change_in_event(&create_key_event, &key_attribs)
-                .ok_or(InvalidInternalState)?;
-
-        let public_key = ProfileChangeHistory::get_change_public_key(create_key_change)?;
+        let public_key = create_key_event.change_block().change().public_key()?;
         let public_key_id = vault.compute_key_id_for_public_key(&public_key).await?;
         let public_key_id = ProfileIdentifier::from_key_id(public_key_id);
 
@@ -114,29 +109,16 @@ impl ProfileState {
     }
 
     pub(crate) async fn get_secret_key_from_event(
-        key_attributes: &KeyAttributes,
         event: &ProfileChangeEvent,
         vault: &mut impl ProfileVault,
     ) -> Result<Secret> {
-        let public_key = ProfileChangeHistory::get_public_key_from_event(key_attributes, event)?;
+        let public_key = event.change_block().change().public_key()?;
 
         let public_key_id = vault.compute_key_id_for_public_key(&public_key).await?;
 
         vault.get_secret_by_key_id(&public_key_id).await
     }
 
-    pub async fn get_root_secret(&mut self) -> Result<Secret> {
-        let public_key = ProfileChangeHistory::get_current_profile_update_public_key(
-            self.change_history().as_ref(),
-        )?;
-
-        let key_id = self
-            .vault
-            .compute_key_id_for_public_key(&public_key)
-            .await?;
-        self.vault.get_secret_by_key_id(&key_id).await
-    }
-
     pub fn has_lease(&self) -> bool {
         self.lease.is_some()
     }
@@ -146,74 +128,75 @@ impl ProfileState {
     }
 }
 
-#[async_trait]
-impl Identity for ProfileState {
-    async fn identifier(&self) -> Result<ProfileIdentifier> {
+impl ProfileState {
+    pub async fn identifier(&self) -> Result<ProfileIdentifier> {
         Ok(self.id.clone())
     }
 
-    async fn create_key(&mut self, label: String) -> Result<()> {
-        let key_attribs = KeyAttributes::new(label);
+    pub async fn create_key(&mut self, label: String) -> Result<()> {
+        let key_attribs = KeyAttributes::default_with_label(label);
+
+        let event = self
+            .make_create_key_event(None, key_attribs, ProfileEventAttributes::new())
+            .await?;
+
+        self.add_change(event).await
+    }
+
+    pub async fn add_key(&mut self, label: String, secret: &Secret) -> Result<()> {
+        let secret_attributes = self.vault.secret_attributes_get(secret).await?;
+        let key_attribs = KeyAttributes::new(
+            label,
+            MetaKeyAttributes::SecretAttributes(secret_attributes),
+        );
 
         let event = {
-            self.create_key(key_attribs, ProfileEventAttributes::new())
+            self.make_create_key_event(Some(secret), key_attribs, ProfileEventAttributes::new())
                 .await?
         };
         self.add_change(event).await
     }
 
-    async fn rotate_profile_key(&mut self) -> Result<()> {
-        let event = {
-            self.rotate_key(
-                KeyAttributes::new(Profile::PROFILE_UPDATE.to_string()),
+    pub async fn rotate_root_secret_key(&mut self) -> Result<()> {
+        let event = self
+            .make_rotate_key_event(
+                KeyAttributes::default_with_label(Profile::ROOT_LABEL.to_string()),
                 ProfileEventAttributes::new(),
             )
-            .await?
-        };
+            .await?;
         self.add_change(event).await
     }
 
     /// Get [`Secret`] key. Key is uniquely identified by label in [`KeyAttributes`]
-    async fn get_profile_secret_key(&self) -> Result<Secret> {
-        self.get_secret_key(Profile::PROFILE_UPDATE.to_string())
-            .await
+    pub async fn get_root_secret_key(&self) -> Result<Secret> {
+        self.get_secret_key(Profile::ROOT_LABEL.to_string()).await
     }
 
-    async fn get_secret_key(&self, label: String) -> Result<Secret> {
-        let key_attributes = KeyAttributes::new(label);
-        let event = ProfileChangeHistory::find_last_key_event(
-            self.change_history().as_ref(),
-            &key_attributes,
-        )?
-        .clone();
-        Self::get_secret_key_from_event(
-            &key_attributes,
-            &event,
-            &mut self.vault.async_try_clone().await?,
-        )
-        .await
+    pub async fn get_secret_key(&self, label: String) -> Result<Secret> {
+        let event =
+            ProfileChangeHistory::find_last_key_event(self.change_history().as_ref(), &label)?
+                .clone();
+        Self::get_secret_key_from_event(&event, &mut self.vault.async_try_clone().await?).await
     }
 
-    async fn get_profile_public_key(&self) -> Result<PublicKey> {
-        self.get_public_key(Profile::PROFILE_UPDATE.to_string())
-            .await
+    pub async fn get_root_public_key(&self) -> Result<PublicKey> {
+        self.get_public_key(Profile::ROOT_LABEL.to_string()).await
     }
 
-    async fn get_public_key(&self, label: String) -> Result<PublicKey> {
-        self.change_history
-            .get_public_key(&KeyAttributes::new(label))
+    pub async fn get_public_key(&self, label: String) -> Result<PublicKey> {
+        self.change_history.get_public_key(&label)
     }
 
     /// Generate Proof of possession of [`Profile`].
     /// channel_state should be tied to channel's cryptographical material (e.g. h value for Noise XX)
-    async fn create_auth_proof(&mut self, channel_state: &[u8]) -> Result<AuthenticationProof> {
-        let root_secret = self.get_root_secret().await?;
+    pub async fn create_auth_proof(&mut self, channel_state: &[u8]) -> Result<AuthenticationProof> {
+        let root_secret = self.get_root_secret_key().await?;
 
         Authentication::generate_proof(channel_state, &root_secret, &mut self.vault).await
     }
     /// Verify Proof of possession of [`Profile`] with given [`ProfileIdentifier`].
     /// channel_state should be tied to channel's cryptographical material (e.g. h value for Noise XX)
-    async fn verify_auth_proof(
+    pub async fn verify_auth_proof(
         &mut self,
         channel_state: &[u8],
         responder_contact_id: &ProfileIdentifier,
@@ -233,7 +216,7 @@ impl Identity for ProfileState {
         .await
     }
 
-    async fn add_change(&mut self, change_event: ProfileChangeEvent) -> Result<()> {
+    pub async fn add_change(&mut self, change_event: ProfileChangeEvent) -> Result<()> {
         let slice = core::slice::from_ref(&change_event);
         if ProfileChangeHistory::check_consistency(self.change_history.as_ref(), slice) {
             self.change_history.push_event(change_event);
@@ -241,12 +224,12 @@ impl Identity for ProfileState {
         Ok(())
     }
 
-    async fn get_changes(&self) -> Result<Changes> {
+    pub async fn get_changes(&self) -> Result<Changes> {
         Ok(self.change_history.as_ref().to_vec())
     }
 
     /// Verify whole event chain of current [`Profile`]
-    async fn verify_changes(&mut self) -> Result<bool> {
+    pub async fn verify_changes(&mut self) -> Result<bool> {
         if !ProfileChangeHistory::check_consistency(&[], self.change_history().as_ref()) {
             return deny();
         }
@@ -274,28 +257,28 @@ impl Identity for ProfileState {
         allow()
     }
 
-    async fn get_contacts(&self) -> Result<Vec<Contact>> {
+    pub async fn get_contacts(&self) -> Result<Vec<Contact>> {
         Ok(self.contacts.values().cloned().collect())
     }
 
-    async fn as_contact(&mut self) -> Result<Contact> {
+    pub async fn as_contact(&mut self) -> Result<Contact> {
         Ok(Contact::new(
             self.id.clone(),
             self.change_history.as_ref().to_vec(),
         ))
     }
 
-    async fn get_contact(&mut self, id: &ProfileIdentifier) -> Result<Option<Contact>> {
+    pub async fn get_contact(&mut self, id: &ProfileIdentifier) -> Result<Option<Contact>> {
         Ok(self.contacts.get(id).cloned())
     }
 
-    async fn verify_contact(&mut self, contact: Contact) -> Result<bool> {
+    pub async fn verify_contact(&mut self, contact: Contact) -> Result<bool> {
         contact.verify(&mut self.vault).await?;
 
         allow()
     }
 
-    async fn verify_and_add_contact(&mut self, contact: Contact) -> Result<bool> {
+    pub async fn verify_and_add_contact(&mut self, contact: Contact) -> Result<bool> {
         if !self.verify_contact(contact.clone()).await? {
             return Err(ContactVerificationFailed.into());
         }
@@ -305,7 +288,7 @@ impl Identity for ProfileState {
         allow()
     }
 
-    async fn verify_and_update_contact(
+    pub async fn verify_and_update_contact(
         &mut self,
         contact_id: &ProfileIdentifier,
         change_events: &[ProfileChangeEvent],
@@ -321,7 +304,7 @@ impl Identity for ProfileState {
             .await
     }
 
-    async fn get_lease(
+    pub async fn get_lease(
         &self,
         _lease_manager_route: &Route,
         _org_id: String,
@@ -335,7 +318,7 @@ impl Identity for ProfileState {
         }
     }
 
-    async fn revoke_lease(&mut self, _lease_manager_route: &Route, lease: Lease) -> Result<()> {
+    pub async fn revoke_lease(&mut self, _lease_manager_route: &Route, lease: Lease) -> Result<()> {
         if let Some(existing_lease) = &self.lease {
             if existing_lease == &lease {
                 self.lease = None
